/**
// This extension was developed by :
// Damian Brun < damianbasic AT gmail DOT com >
//
// Licence: GPLv2+
**/

 
const AppDisplay = imports.ui.appDisplay;
const PopupMenu = imports.ui.popupMenu;

const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;


function setQuicklist (appIconMenu) {
    if (appIconMenu._source.app.is_window_backed()) return;

    let keyfile = null;
    let ayatana = false;

    function item_new_from_action(action) {
        let item = null;
        let group = null;

        if (ayatana) 
            group =  action + ' Shortcut Group';
        else
            group = 'Desktop Action ' + action;

        let name = null;
        let exec = null;

        let lang = GLib.get_language_names();
        for (let i=0; i<lang.length; i++) {
            try {name = keyfile.get_locale_string(group, 'Name', lang[i]);} catch (e) {}
            if (name != null) break;
        }
        if (name == null) {
            try {name = keyfile.get_string(group, 'Name');} catch(e) {}
        }

        try {
            exec = keyfile.get_string(group, 'Exec');
        } catch (e) {}

        if ( (name != null) && (exec != null) ){
            item = new PopupMenu.PopupMenuItem(name);
                item.connect('activate', function() {
                try {GLib.spawn_command_line_async(exec);} catch (e) {}
            });
        }

        return item;
    }


    function get_actions() {
        let actions = null;

        try {
            actions = keyfile.get_string_list('Desktop Entry', 'Actions');
        } 
        catch(e) {
            ayatana = true;
        }

        if (actions == null) {
            try {
                actions = keyfile.get_string_list('Desktop Entry', 'X-Ayatana-Desktop-Shortcuts');
            }
            catch(e) { 
                ayatana = false; 
            }
        }

        return actions;
    }


    let app = new Gio.DesktopAppInfo.new(appIconMenu._source.app.get_id());
    let desktopFile = app.get_filename();
    
    if (GLib.file_test(desktopFile, GLib.FileTest.EXISTS) == false) 
        return;

    keyfile = new GLib.KeyFile();

    if (keyfile.load_from_file(desktopFile, GLib.KeyFileFlags.NONE) == true) {
        let actions = get_actions();

        if (actions != null) {
            let n = 0;

            for (let i=0; i<actions.length; i++) {
                let item = item_new_from_action(actions[i]);

                if (item != null) {
                    appIconMenu.addMenuItem(item, n);
                    n = n + 1;
                }
            }

            if (n != 0) {
                let item = new PopupMenu.PopupSeparatorMenuItem();
                appIconMenu.addMenuItem(item, n);  
            }
        }
    }

    delete keyfile;
}


function init() {

}

let origAppIconMenu_redisplay = null;

function enable() {
    origAppIconMenu_redisplay = AppDisplay.AppIconMenu.prototype._redisplay;

    AppDisplay.AppIconMenu.prototype._redisplay = function() {
        origAppIconMenu_redisplay.call(this);
        setQuicklist(this);
    };
}

function disable() {
    AppDisplay.AppIconMenu.prototype._redisplay = origAppIconMenu_redisplay;
}

